


#pragma CODE_STATE(rail_voltage_monitor, 32)  // 16 = thumb mode, 32 = ARM mode
void rail_voltage_monitor(int32 adc_reading, Uint8 number)
{
    static VMON_STATES vmon_state_ad05 = VMON_RAIL_DISABLED;

    switch(vmon_state_ad05)
    {
        case VMON_RAIL_DISABLED:
		{
            if(rail_para_const[number].on_off_ctrl==1)
            {
                rail[rail_para_const[number].rail_index].fault.bit.uv = 0;          // Clear UV fault flag for the rail 
                rail[rail_para_const[number].rail_index].fault.bit.ov = 0;          // Clear OV fault flag for the rail
            }
            if (rail[rail_para_const[number].rail_index].enabled)
            {
                if (dc_ok_delay_counter < rail_para_const[number].rampup_blanking_cnt)
                {
                    // update dc_ok_delay_counter
                    dc_ok_delay_counter = rail_para_const[number].rampup_blanking_cnt;
                }
                vmon_state_ad05 = VMON_RAIL_RAMPUP; // next state
            }
            break;
		}

        case VMON_RAIL_RAMPUP:
		{
            // rail is coming up, ignore faults in this state
            if (rail[rail_para_const[number].rail_index].enabled == 0)
            {
                // rail has become disabled
                vmon_state_ad05 = VMON_RAIL_DISABLED;   // next state
            }
            else
            {
                if (dc_ok_delay_counter == 0)
                {
                    // fault blanking timer has elapsed
                    // check Vout against OV and UV thresholds
                    if (adc_reading > rail_para_const[number].rail_ovp_threshold)
                    {
                        // OV fault
                        // monitored voltage is greater than OVP threshold
                        rail[rail_para_const[number].rail_index].fault.bit.ov = 1;      // set OV fault flag for the rail
                        set_dc_nok_flag(rail_para_const[number].rail_id);               // set DC_NOK flag for the rail

                        // if the event log buffer is not being used (we know this if event_log_update_flag = 0), then initiate update of the event log in data flash
                        if (event_log_update_flag == 0)
                        {
                            update_event_log_buffer(rail_para_const[number].rail_index);
                        }

                        vmon_state_ad05 = VMON_RAIL_OV_FAULT;       // next state
                        
                    }
                    else if (adc_reading < rail_para_const[number].rail_uvp_threshold)
                    {
                        // UV fault
                        // monitored voltage is less than UVP threshold
                        rail[AD05_RAIL_INDEX].fault.bit.uv = 1;             // set UV fault flag for the rail
                        set_dc_nok_flag(rail_para_const[number].rail_id);                      // set DC_NOK flag for the rail

                        // if the event log buffer is not being used (we know this if event_log_update_flag = 0), then initiate update of the event log in data flash
                        if (event_log_update_flag == 0)
                        {
                            update_event_log_buffer(rail_para_const[number].rail_index);
                        }

                        vmon_state_ad05 = VMON_RAIL_UV_FAULT;       // next state
                    }
                    else
                    {
                        // no fault, DC is OK for this rail, so clear DC_NOK flag
                        clear_dc_nok_flag(rail_para_const[number].rail_id);
                        vmon_state_ad05 = VMON_RAIL_DC_OK;  // next state
                    }
                }
            }
            break;
		}

        case VMON_RAIL_DC_OK:
		{
            // rail is enabled, and is between it's OV and UV levels
            if (rail[rail_para_const[number].rail_index].enabled == 0)
            {
                // rail has become disabled
                vmon_state_ad05 = VMON_RAIL_DISABLED;   // next state
            }
            else
            {
                // check Vout against OV and UV thresholds
                if (adc_reading > rail_para_const[number].rail_ovp_threshold)
                {
                    // OV fault
                    // monitored voltage is greater than OVP threshold
                    rail[rail_para_const[number].rail_index].fault.bit.ov = 1;      // set OV fault flag for the rail
                    set_dc_nok_flag(rail_para_const[number].rail_id);               // set DC_NOK flag for the rail
                    
                    // if the event log buffer is not being used (we know this if event_log_update_flag = 0), then initiate update of the event log in data flash
                    if (event_log_update_flag == 0)
                    {
                        update_event_log_buffer(rail_para_const[number].rail_index);
                    }

                    vmon_state_ad05 = VMON_RAIL_OV_FAULT;       // next state
                }
                else if (adc_reading < rail_para_const[number].rail_uvp_threshold)
                {
                        // UV fault
                        // monitored voltage is less than UVP threshold
                    rail[rail_para_const[number].rail_index].fault.bit.uv = 1;             // set UV fault flag for the rail
                    set_dc_nok_flag(rail_para_const[number].rail_id);                      // set DC_NOK flag for the rail

                    // if the event log buffer is not being used (we know this if event_log_update_flag = 0), then initiate update of the event log in data flash
                    if (event_log_update_flag == 0)
                    {
                        update_event_log_buffer(rail_para_const[number].rail_index);
                    }

                    vmon_state_ad05 = VMON_RAIL_UV_FAULT;       // next state
                }
            }
            break;
		}

        case VMON_RAIL_OV_FAULT:
		{
            // OV fault has occured
            if (rail[rail_para_const[number].rail_index].enabled == 0)
            {
                // rail has been disabled
                clear_dc_nok_flag(rail_para_const[number].rail_id);             // clear DC_NOK flag for the rail
                rail[rail_para_const[number].rail_index].fault.bit.ov = 0;      // clear OV fault flag for the rail
                vmon_state_ad05 = VMON_RAIL_DISABLED;       // next state
            }
            else
            {
                // rail is still enabled
                // check if Vout < OV threshold
                if (adc_reading < rail_para_const[number].rail_ovp_threshold)
                {
                        // no fault, DC is OK for this rail, so clear DC_NOK flag
                   	clear_dc_nok_flag(rail_para_const[number].rail_id);
                    rail[rail_para_const[number].rail_index].fault.bit.ov = 0;      // clear OV fault flag for the rail
                    vmon_state_ad05 = VMON_RAIL_DC_OK;         // next state
                }
            }
            break;
		}

        case VMON_RAIL_UV_FAULT:
		{
            // UV fault has occured
            if (rail[rail_para_const[number].rail_index].enabled == 0)
            {
                // rail has been disabled
                clear_dc_nok_flag(rail_para_const[number].rail_id);             // clear DC_NOK flag for the rail
                rail[rail_para_const[number].rail_index].fault.bit.uv = 0;      // clear UV fault flag for the rail
                vmon_state_ad05 = VMON_RAIL_DISABLED;       // next state
            }
            else
            {
                // rail is still enabled
                // check if Vout > UV threshold
                if (adc_reading > rail_para_const[number].rail_uvp_threshold)
                {
                        // no fault, DC is OK for this rail, so clear DC_NOK flag
                  	clear_dc_nok_flag(rail_para_const[number].rail_id);
                    rail[rail_para_const[number].rail_index].fault.bit.uv = 0;      // clear UV fault flag for the rail
                    vmon_state_ad05 = VMON_RAIL_DC_OK;         // next state
                }
            }
            break;
		}

        default:
            break;
    }

}